import os
import re
import subprocess
from datetime import datetime

csv_headers = [
        'minute',
        'hour',
        'day',
        'month',
        'year',
        'description',
        'people',
        'location',
        'tags',
        'file',
        'call',
        'text']

all_months = ['January',
            'February',
            'March',
            'April',
            'May',
            'June',
            'July',
            'August',
            'September',
            'October',
            'November',
            'December']

class CSVReader(object):
    """
    Reads CSV files containing events
    and parses the information.
    This is used by Timeline and ScrapeTimeline,
    so it is put in a common class
    that is extended by both.
    """
    def load_csv_into_memory(self,csv_file):
        
        self.all_dat = []

        self.csv_file = csv_file

        # if file does not exist, touch and populate with header row.
        # then continue.
        if os.path.isfile(csv_file) <> True: 

            print "File "+csv_file+" does not exist. Creating..."
            subprocess.call(["touch",csv_file])

            # write the header so pop works
            with open(self.csv_file,'w') as f:
                header = ",".join(csv_headers)
                print >> f,header

        # Now our csv file exists, 
        # load CSV into memory as a list

        lines = [line.rstrip() for line in open(csv_file)]

        # Skip over first line (headers)
        try:
            header_keys = lines.pop(0).split(",")
        except:
            raise Exception("Error doing pop, your file may be missing a header. If so, just delete it.")


        # Load all other lines
        all_dat = []
        for j,line in enumerate(lines):
            d = {}
            tokens = line.split(",")
            for key, val in zip(header_keys,tokens):
                # ugh. deal with Excel/OpenOffice double-quoting strings
                key = re.sub('"','',key)
                val = re.sub('"','',val)
                d[key] = val

            # now that d contains all keys and values associated with this event,
            # let's make sure the values are all valid
            #self.error_checking(d)

            # Put event in master event list
            if d not in all_dat:
                all_dat.append(d)

        self.all_dat = all_dat




    #def error_checking(self,d):
    #    """
    #    Do error checking for event dict

    #    0 <= Minute <= 59
    #    0 <= Hour <= 23
    #    0 <= Day <= 31
    #    1 <= Month <= 12
    #    """
    #    #self.numerical_assert_between(d,'minute',0,59)
    #    #self.numerical_assert_between(d,'hour',0,23)
    #    #self.numerical_assert_between(d,'day',1,31)
    #    #self.numerical_assert_between(d,'month',1,12)



    #def numerical_assert_between(self,d,key,lo,hi):
    #    """
    #    Assert that key `key` in dictionary `d` 
    #    is between `lo` and `hi`
    #    """
    #    # first of all, cast strings as numbers
    #    if d[key] == '':
    #        d[key] = 0
    #    try:
    #        d[key] = int(d[key])
    #    except ValueError:
    #        print d[key]
    #        d[key] = float(d[key])

    #    key_not_empty = d[key]<>None
    #    key_under_max = d[key] <= hi
    #    key_over_min  = d[key] >= lo
    #    try:
    #        assert(key_not_empty)
    #        assert(key_under_max)
    #        assert(key_over_min)
    #    except AssertionError:
    #        print d
    #        raise Exception()



    def export_csv(self):
        """
        We are done adding all of our events,
        so now export the new csv file.
        """
        # this makes testing a helluva lot easier
        self.sneak_in_a_test_call_line()

        with open(self.csv_file,'w') as f:
            header = ",".join(csv_headers)
            print >> f,header
            for line in self.all_dat:
                print >> f,line

    
    def sneak_in_a_test_call_line(self):
        d = {} 
        d['minute']         = 0
        d['hour']           = 12
        d['day']            = datetime.now().day   
        d['month']          = datetime.now().month 
        d['year']           = 1970
        d['description']    = 'this is a dummy description'
        d['people']         = ''
        d['location']       = ''
        d['tags']           = ''
        d['file']           = 'http://charlesreid1.com/twilio.mp3'
        d['call']           = 1
        d['text']           = 0

        # this mess of a list comprehension
        # just stitches the dict into a CSV line
        self.all_dat.append( \
                ','.join( \
                    [str(d[key]).encode('utf8')  \
                        for key in csv_headers]) \
                )

