from numpy import logical_and
from bs4 import BeautifulSoup
from urllib import *
from os.path import basename
import re

from CSVReader import *

"""
Scrapes the webpage
Assembles the CSV
Creates an anticipated MP3 link 
Passes the MP3 filename to Uploader
"""

class Scraper(CSVReader):

    def __init__(self,csv_file):
        self.load_csv_into_memory(csv_file)
        self.base_url = 'http://nixontapes.org/'
        
        # people lookup from above website
        self.Persons = { 
            'P' : 'Richard Nixon',
            'GHWB' : 'George H.W. Bush',
            'GRF' : 'Gerald Ford',
            'HAK' : 'Henry Kissinger',
            'HRH' : 'H.R. Haldeman',
            'LBJ' : 'Lyndon Johnson',
            'RJD' : 'Bob Dole',
            'RWR' : 'Ronald Reagan'
        }

    def scrape_nixontapes(self):
        """
        Scrape nixontapes.org/presidents.html
        to build a little CSV database of files 
        and associated metadata
        """
        base_url = self.base_url
        Persons = self.Persons

        html_files = ['presidents.html']

        full_html_files = ['files/'+html_file for html_file in html_files]

        # This will be a list of dictionaries
        # each dict contains one tape recording

        for html_file in full_html_files:
        
            soup = BeautifulSoup(open(html_file))
        
            # here we're just going to do 
            # some more sophisticated parsing
        
            # this may change page-to-page,
            # in which case you can just put a 
            # case structure based on html_file name
        
            our_table = None
            for table in soup.find_all('table'):
        
                try:
                    if logical_and(table['border']=='1',table['width']=='652'):
                        our_table = table
                except:
                    pass
        
            for j,row in enumerate(our_table.find_all('tr')):
                if j>0:
        
                    cols = row.find_all('td')
        
                    conversation    = cols[0].find('font').get_text()
                    date            = cols[1].find('font').get_text() 
        
                    # dates are MM/DD/YYYY
                    exp = re.compile('([0-9]{2})\/([0-9]{2})\/([0-9]{4})')
                    m = exp.search(date)
                    tape_dates = m.groups()
        
                    # boom
                    tape_min   = 0
                    tape_hour  = 0
                    tape_day   = tape_dates[0]
                    tape_month = tape_dates[1]
                    tape_year  = tape_dates[2]
        
                    summary_link = cols[6].find('a').get('href')
        
                    # boom
                    tape_description = base_url + summary_link
        
                    tape_people = []
                    participants = cols[3].find('font').get_text()
                    for p in Persons.keys():
                        if p in participants:
                            full_name = Persons[p]
                            # boom
                            tape_people.append(full_name)

                    tape_people = ";".join(tape_people)
        
                    tape_location = ''
                    tape_tags = ''
                    tape_file = base_url + cols[5].find('a').get('href') 
                    tape_call = 1
                    tape_text = 0
        
        
                    # now shove all this info into a dictionary
                    # that goes into a csv file
                    d = {} 
                    d['minute']         = tape_min 
                    d['hour']           = tape_hour
                    d['day']            = tape_day
                    d['month']          = tape_month 
                    d['year']           = tape_year
                    d['description']    = tape_description
                    d['people']         = tape_people
                    d['location']       = tape_location
                    d['tags']           = tape_tags
                    d['file']           = tape_file
                    d['call']           = tape_call
                    d['text']           = tape_text


        
                    # this mess of a list comprehension
                    # just stitches the dict into a CSV line
                    self.all_dat.append( \
                            ','.join( \
                                [str(d[key]).encode('utf8')  \
                                    for key in csv_headers]) \
                            )

        self.export_csv()

if __name__=="__main__":
    s = Scraper('WatergateTimeline.csv')
    s.scrape_nixontapes()

